/*
bag-control.dm

This file handles mob's inventories.  They are universal, meaning they
can be used by NPCs for shops, or Enemies for weapons and drops.

Key things to know:
	- New(mob/m, num = 20)
		: m is the mob it is attached to, and num is the amount of
			items the bag can hold.

	- amount()
		: this returns how many items are in the bag in total.

	- add_item(item/i, amount, truncate = TRUE)
		: i is the item to be added.
		: amount is how many of i will be added.
		: truncate is TRUE by default.  When it is true, the proc
			will automatically reduce amount to make the item fit if
			there isn't enough room.

	- remove_item(item/i, amount)
		: i is the item to be removed.
		: amount is how many of item will be removed.

	- empty()
		: this proc will empty the bag.  if the bag is attached to a client,
			it will simply delete the contents.  otherwise, it will drop
			the contents at the owner's location.  an issue here is that
			the maptext will show for all of the items.  empty() is probably
			one of the first, if any, procs you should mess with too much in
			here.
*/

bag_control
	var
		mob/owner

		slots

		list/contents = list()

	New(mob/m, num = 20)
		..()

		// If there is no owner, there's no point to the bag.
		if(!m)
			del src

		owner = m
		slots = num

	proc

		amount()
			. = 0
			for(var/item/i in contents)
				. += contents[i]



		add_item(item/i, amount, truncate = TRUE)

			var/fullness = amount()

			// If the bag is already full, nothing else can fit.
			if(fullness >= slots)
				return FALSE

			// If this amount of i will overflow the bag, either stop or truncate the amount to make it fit.
			if(fullness + amount > slots)
				if(!truncate)
					return FALSE

				else
					amount = slots - fullness

			// If it's already in the bag, just add to it.
			var/found = FALSE
			for(var/item/types in contents)
				if(i.type == types.type)
					found = TRUE
					contents[types] += amount

			// If not, then make a new addition.
			if(!found)
				contents[i] = amount

			// Store the item itself in the mob's contents, just to keep it safe.
			owner.contents += i
			i.owner = owner

		remove_item(item/i, amount)

			// We can't remove an item we don't have.
			if(!(i in contents))
				return FALSE

			// Just a safety check.  We don't want to remove more than we've got.
			if(contents[i] < amount)
				amount = contents[i]

			contents[i] -= amount

			// If theres none left, remove the item completely.
			if(contents[i] <= 0)
				contents -= i

				owner.contents -= i
				del i

		empty()
			if(!contents.len)
				return FALSE

			if(owner.client)
				for(var/item/i in contents)
					remove_item(i, contents[i])

			else
				var/amount = 0
				for(var/item/i in contents)
					amount = contents[i]
					if(!amount)
						continue

					for(var/v = 1 to amount)
						new i.type(owner.loc)