/*
procs.dm

This file tells all of the procs found in _procs.dm what to do.

Descriptions:
	- initiate()
		: this is called in Login().  this is just the initialization for clients.

	- pickup_item(dist = 1)
		: dist is how far away it will search for an item to pick up.
			it uses oview(), meaning it counts things in src's location.

	- equip_item()
		: this produces a list of items to equip.  To equip a single item,
			use item.equip()

	- unequip_item()
		: produces a list of equipped items to unequip.  To unequip a single item,
			use item.unequip()

	- update_inventory()
		: this updates a client's inventory control.  remember to call it whenever
			an item is equipped, unequipped, added, or taken.

	- reset_maptext()
		: this updates a client's maptext to display messages depending on what's around them.

	- attack(atom/a)
		: this causes src to attack a.  It is well documented step-by-step below.

	- death_check(mob/killer)
		: this checks to see if src is dead.  in the attack() proc, it is called like so:
		: a.death_check(src)
		: the code is very self-explanatory, though undocumented.

	- reset()
		: this is the proc called when a client dies.  it resets them.  you could also call it "reviving" them.

	- get_spawn()
		: this returns the location of the player /spawnArea.  Used in reset() to send the player to a certain location
			and in Login().

	- base_EventCycle()
		: this is just used for the cooldowns.  it is a "background timer".  You'll need Deadron's EventLoop library.
*/

mob

	initiate()
		bag = new(src)
		cooldowns["attack"] = 0
		client.focus = src

	pickup_item(dist = 1)
		for(var/gold/g in oview(dist))
			gold += g.amount
			src << "Picked up <b>[g.amount]</b> gold."
			del g

		for(var/item/i in oview(dist))
			bag.add_item(i, 1)
			update_inventory()
			src << "Picked up <b>[i]</b>."

	equip_item()
		if(!(bag.contents.len))
			return FALSE

		var/list/l = list()
		for(var/item/i in bag.contents)
			l += i

		var/item/itm = input("Which item would you like to equip?", "Equip item") in l
		itm.equip()
		update_inventory()

	unequip_item()
		if(!(bag.contents.len))
			return FALSE

		var/list/l = list()
		for(var/item/i in bag.contents)
			l += i

		var/item/itm = input("Which item would you like to unequip?", "Unequip item") in l
		itm.unequip()
		update_inventory()

	update_inventory()
		if(!client)
			return FALSE

		// First reset the inventory.
		winset(src, "default.inventory_grid", "cells = 0x0")

		src << output("Bag", "default.inventory_grid:1,1")
		src << output("Status: [bag.amount()]/[bag.slots] Items", "default.inventory_grid:2,1")

		if(!(bag.contents.len))
			return FALSE

		// Now refill it.
		var/item/i
		for(var/v = 1 to bag.contents.len)
			i = bag.contents[v]
			var/amnt = bag.contents[i]
			//if(i.equipped)
			//	amnt -= 1
			if(amnt > 0)
				src << output(i, "default.inventory_grid:1,[v + 1]")
				src << output("x[amnt][i.equipped ? "    \[Equipped\]" : ""]", "default.inventory_grid:2,[v + 1]")

	reset_maptext()
		if(!client)
			return FALSE

		maptext_Screen.Cut()
		maptext_UpdateScreen()
		for(var/atom/v in view(1))
			if(istype(v, /npc))
				v.maptext_CreateText(src, "[v.name]: Press Space to interact.")
			if(istype(v, /ai))
				v.maptext_CreateText(src, "[v.name]: Run into me to attack.")
			if(istype(v, /sign))
				v.maptext_CreateText(src, "[v:message]", FLY_LAYER, v:mt_width, v:mt_height, v:px, v:py)
			if(istype(v, /gold))
				v.maptext_CreateText(src, "Gold: Press G to pick up.")
			if(istype(v, /item))
				v.maptext_CreateText(src, "[v.name]: Press G to pick up.")

	attack(atom/a)
		// This is going to be called when you Bump() a mob.  This isn't hack-and-slash,
		// just bump into a mob to attack it.  You can't spam it though, there is
		// a cooldown of 5 ticks (half of a second).


		if(!a || !ismob(a))
			return FALSE

		var/mob/m = a

		// Don't want enemies attacking enemies, just you
		if(istype(src, /ai))
			if(istype(m, /ai))
				return FALSE

		// Don't want to attack NPCs
		if(m.NPC)
			return FALSE

		if(cooldowns["attack"] > 0)
			return FALSE

		// Now that the safety checks are done, we go to the guts.
		var/hit = TRUE

		// If they miss due to their accuracy
		if(!(prob(accuracy * 100)))
			hit = FALSE

		// If they miss due to m's agility
		if(prob((accuracy - (m.agility/2)) * 100))
			hit = FALSE

		// Let's call their attacked() proc.  Does nothing by default, define it for AI
		m.attacked(src)

		// If they hit...
		if(hit)

			// Do some damage calculation.
			var/damage = round(strength - (m.defense/2))
			if(damage <= 0)
				damage = 1

			m.hit_points -= damage
			m.death_check(src)

			// This is just a safety check to keep from sending messages for mobs that are dying.
			if(!m || m.dying)
				return FALSE

			src << "Hit <b>[m]</b> for <b>[damage]</b> damage!"
			m << "<b>[src]</b> hit you for <b>[damage]</b> damage!"

		else
			src << "You missed <b>[m]</b>!"

		cooldowns["attack"] += 7

	death_check(mob/killer)
		if(hit_points > 0)
			return FALSE

		dying = TRUE

		if(killer.client)
			killer << "You killed <b>[src]</b>!"

		for(var/item/i in bag.contents)
			i.unequip()

		if(!client)
			if(killer.client)
				killer.reset_maptext()
			if(gold)
				var/gold/g = new(gold, FALSE)
				g.loc = src.loc

			bag.empty()
			del src

		else
			src << "You <b>died</b>!"
			reset()

	reset()
		var/spawnArea/s = get_spawn()
		loc = locate(s.x, s.y, s.z)
		gold = 0
		hit_points = 50
		dying = FALSE
		bag.empty()
		for(var/ai/a)
			if(a.target == src)
				a.target = null

	get_spawn()
		return locate(/spawnArea)




	base_EventCycle()
		for(var/v in cooldowns)
			cooldowns[v] -= 1
			if(cooldowns[v] < 0)
				cooldowns[v] = 0